svn commit: r285678 - head/sys/dev/mmc

Ian Lepore ian at FreeBSD.org
Sat Jul 18 16:56:52 UTC 2015


Author: ian
Date: Sat Jul 18 16:56:51 2015
New Revision: 285678
URL: https://svnweb.freebsd.org/changeset/base/285678

Log:
  Deselect the sd card before re-selecting it when working around a problem
  with some cards that causes them to become deselected after probing for
  switch capabilities.  The old workaround fixes the behavior with some cards,
  but causes problems with the cards the behave correctly and don't become
  deselected.  Forcing a deselect then reselect appears to work correctly
  with all cards in initial testing.

Modified:
  head/sys/dev/mmc/mmc.c

Modified: head/sys/dev/mmc/mmc.c
==============================================================================
--- head/sys/dev/mmc/mmc.c	Sat Jul 18 15:28:32 2015	(r285677)
+++ head/sys/dev/mmc/mmc.c	Sat Jul 18 16:56:51 2015	(r285678)
@@ -1381,17 +1381,18 @@ mmc_discover_cards(struct mmc_softc *sc)
 			}
 
 			/*
 			 * We reselect the card here. Some cards become
 			 * unselected and timeout with the above two commands,
 			 * although the state tables / diagrams in the standard
 			 * suggest they go back to the transfer state. The only
 			 * thing we use from the sd_status is the erase sector
 			 * size, but it is still nice to get that right. It is
 			 * normally harmless for cards not misbehaving. The
 			 * Atmel bridge will complain about this command timing
 			 * out. Others seem to handle it correctly, so it may
 			 * be a combination of card and controller.
 			 */
+			mmc_select_card(sc, 0);
 			mmc_select_card(sc, ivar->rca);
 			mmc_app_sd_status(sc, ivar->rca, ivar->raw_sd_status);
 			mmc_app_decode_sd_status(ivar->raw_sd_status,
@@ -1381,17 +1381,18 @@ mmc_discover_cards(struct mmc_softc *sc)
 
 			/*
-			 * We reselect the card here. Some cards become
-			 * unselected and timeout with the above two commands,
-			 * although the state tables / diagrams in the standard
-			 * suggest they go back to the transfer state. The only
-			 * thing we use from the sd_status is the erase sector
-			 * size, but it is still nice to get that right. It is
-			 * normally harmless for cards not misbehaving. The
-			 * Atmel bridge will complain about this command timing
-			 * out. Others seem to handle it correctly, so it may
-			 * be a combination of card and controller.
+			 * We deselect then reselect the card here.  Some cards
+			 * become unselected and timeout with the above two
+			 * commands, although the state tables / diagrams in the
+			 * standard suggest they go back to the transfer state.
+			 * Other cards don't become deselected, and if we
+			 * atttempt to blindly re-select them, we get timeout
+			 * errors from some controllers.  So we deselect then
+			 * reselect to handle all situations.  The only thing we
+			 * use from the sd_status is the erase sector size, but
+			 * it is still nice to get that right.
 			 */


More information about the svn-src-all mailing list